% system, constraints and target set
% horizon N
% diskretization interval h
% system - x+ = A x + B u + G w
% output y = C x + D w + H w + v
% constraints -  X, U = {x, u | C x + D u + E eps \leq e }



system.h = 0.08;
system.n = 7;
system.m = 2;

A = [0 1    0        0       0        0;
     0 0 -K_2        0       0        0;
     0 0    0        1       0        0;
     0 0 -K_1*K_pp -K_1*K_pd 0        0;
     0 0    0        0       0        1;
     0 0    0        0  -K_3*K_ep -K_3*K_ed];
B =    [0           0;
        0           0;
        0           0;
        K_1*K_pp    0;
        0           0;
        0        K_3*K_ep];

A_d = system.h * [0;
                  1; 
                  0; 
                  0; 
                  0; 
                  0];

E = [A_d; 0];

% discretize helicopter equations   
system.A_heli = system.h * A + eye(system.n-1);
system.B_heli = system.h * B;    

% system with modelled disturbance
system.A = [system.A_heli       A_d; ...
            zeros(1,system.n-1) 1];
system.B = [system.B_heli; 0 0];    
system.G = eye(system.n);

% output with modelled disturbance
system.C = [1 0 0 0 0 0 0;
            0 1 0 0 0 0 0;
            0 0 1 0 0 0 0;
            0 0 0 1 0 0 0;
            0 0 0 0 1 0 0;
            0 0 0 0 0 1 0];
system.D = zeros(system.n-1, system.m);
system.H = zeros(system.n-1, system.n);

% constraints
c_travel          = 210;         % for travel
c_pitch           = 30;          % for pitch
c_pitch_ref       = 25;          % for pitch ref

constraints.C = [1 0 0 0 0 0 0; 
                 -1 0 0 0 0 0 0; 
                 0 0 1 0 0 0 0; 
                 0 0 -1 0 0 0 0;  
                 zeros(system.m, system.n)];
                 
constraints.D = [ zeros(size(constraints.C,1) - system.m, system.m) ;
                  1 0; 
                 -1 0];
              
constraints.e = DEGTORAD * [c_travel;         c_travel; 
                            c_pitch;          c_pitch; 
                            c_pitch_ref;      c_pitch_ref];

constraints.E       = [0;
                       0;
                      -1;
                      -1;
                       0;
                       0];                     

% disturbance
% used when simulating the disturbance
disturbance.E       = [1; -1];
dist_low_bd         = 3;
dist_up_bd          = 3.6;
disturbance.g       = [dist_up_bd * DEGTORAD; -dist_low_bd * DEGTORAD];
