
set(groot, 'defaultAxesTickLabelInterpreter','latex'); set(groot, 'defaultLegendInterpreter','latex');
set(0,'defaultTextInterpreter','latex'); %trying to set the default
clear all
%close all

%% parameters
system.Nsim     = 800; % simulation time
system.N        = 15;  % control horizon
system.stable   = 0;   % enable terminal cost and terminal constraint
with_estimator  = 1;   % enable kalman filter
run_sim         = 0;   % enable simulation
run_heli        = 1;   % enable code generation
%% load system data
init;
data;

%% MPC objective function
% weighting
% system states
q_travel          = 3.8;     % for travel
q_travel_rate     = 2.8;     % for travel rate
q_pitch           = 0.9;     % for pitch
q_pitch_rate      = 1.2;     % for pitch rate
q_elevation       = 2;       % for elevation
q_elevation_rate  = 1;       % for elevation rate
q_dist            = 0;       % for modelled disturbance
% control inputs
r_pitch_ref       = 1.2;     % for pitch ref
r_elevation_ref   = 1;       % for elevation ref
cost.S            = 0.3;     % for slack variable

%generate matrices Q, Q_heli, R
cost.Q = diag([q_travel    q_travel_rate    ...
               q_pitch     q_pitch_rate     ...
               q_elevation q_elevation_rate ...
               q_dist]);
cost.R = diag([r_pitch_ref r_elevation_ref]);

%% kalman filter
% create matrices Q_K, R_K
kalman_filter;

%% offline calculations
offline_calc;

%% assign to problem
problem.system                              = system;
problem.constraints                         = constraints;
problem.cost                                = cost;
problem.estimator                           = estimator;
problem.disturbance                         = disturbance;
[problem.mpc_cost, problem.mpc_constraints] = gen_mpc_matrices(problem); % generate matrices for mpc

%% initial value
travel_offset     = -50;   % for travel
elevation_offset  = -30;   % for elevation
L_model           = L_K;   % assign gain to estimator in model

if (run_heli)
%% generate code for S-Function
% make sure all files in the 'c_code'-directory are closed
gen_code_osqp;         % generate code based on problem data
gen_code_params;       % generate strings for bin, cin, f, Su, Sx, 
                       % NUM_CONSTRAINTS, LENGTH_Z, LENGTH_X, LENGTH_U for 'workspace.h'
                       
%% build Simulink model
% build S-Function builder                   
%blkHandle   = get_param('helicopter_mpc/MPC/S-Function Builder', 'Handle');
%appdata     = sfunctionwizard(blkHandle, 'GetApplicationData');
%appdata     = sfunctionwizard(blkHandle, 'Build', appdata);

% build and connect Simulink model via QUARC
%qc_build_model('helicopter_mpc')
%qc_connect_model('helicopter_mpc')
end

if (run_sim)

%% disturbance
w_sequence = gen_disturbance(problem);
    
%% initial values
system.x0         = [   travel_offset * DEGTORAD; 0; 
                                               0; 0; 
                     elevation_offset * DEGTORAD; 0; 
                                                  0];
x(:,1) = system.x0;
x_est(:,1) = system.x0;

%% simulation
for i = 1 : system.Nsim
    % with kalman filter
    if (with_estimator)
        optimal(i)       = online_calc(problem,x_est(:,i));
        u(:,i)           = optimal(i).u(:,1);
        x_opt(:,i)       = optimal(i).x(:,1);
        e_flag(i)        = optimal(i).e;
        run_time_sim(i)  = optimal(i).run_time;
        x_est(:,i+1)     = problem.system.A * x_est(:,i) + problem.system.B * u(:,i) ...
                         + L_K * system.C * (x(:,i) - x_est(:,i));
        if ~(i == system.Nsim)
                x(:,i+1) = problem.system.A * x(:,i) + problem.system.B * u(:,i) ...
                          + E * w_sequence(:,i);
        end  
    % without kalman filter   
    else
        optimal(i)       = online_calc(problem,x(:,i));
        u(:,i)           = optimal(i).u(:,1);
        x_opt(:,i)       = optimal(i).x(:,1);
        e_flag(i)        = optimal(i).e;   
        run_time_sim(i)  = optimal(i).run_time;        
        if ~(i == system.Nsim)
                x(:,i+1) = problem.system.A * x(:,i) + problem.system.B * u(:,i) ...
                         + E * w_sequence(:,i);
        end  
    end  
end
end

if (run_sim)
    plot_sim;
    plot_run_time_sim;
    if (with_estimator)
        plot_estimator_sim;
    end    
    mean(x(1,system.Nsim - 100:system.Nsim))
end    


