% get optimal decision variable and optimal value
%OSQP: min   0.5 x' * H * x + (x_0' * f)' * x
%      s.t   -Inf \leq Ain \leq bin + cin * x0

n    = problem.system.n;
m    = problem.system.m;
N    = problem.system.N;

f    = problem.mpc_cost.f;
bin  = problem.mpc_constraints.bin;
cin  = problem.mpc_constraints.cin;

x0_osqp = [1; 1; 1; 1; 1; 1; 1];

H    = problem.mpc_cost.H;
q    = (x0_osqp' * f)';
A    = [problem.mpc_constraints.Ain];  
lb   = -Inf * ones(size(problem.mpc_constraints.Ain,1), 1);
ub   = bin + cin * x0_osqp;

%% set up osqp problem
qp_problem        = osqp;                           % create osqp object
settings          = qp_problem.default_settings();  % obtain defualt settings
settings.verbose  = 0;                              % disable printing output
qp_problem.setup(   H, ...
                    q, ...             
                    A, lb, ub, ...
                    settings);

 %% generate code 
dir_name = 'c_code';  % if changing this name, remember to change the name in 'S-Function Builder'
qp_problem.codegen(dir_name,                       ...
                   'project_type',   'Makefile',   ...
                   'parameters',     'matrices',   ...
                   'force_rewrite',  true);

%% move files that are generated 
current_path = pwd;

file_names = {'auxil.h',           'constants.h',   'glob_opts.h',  ...
              'kkt.h',             'lin_alg.h',     'osqp.h',       ...
              'osqp_configure.h',  'proj.h',        'qdldl.h',       ...
              'qdldl_interface.h', 'qdldl_types.h', 'scaling.h',    ...
              'types.h',           'util.h',        'workspace.h' };

for i = 1:size(file_names,2)
    copyfile( [current_path '\' dir_name '\include\'  char(file_names(i))], ...
              [current_path '\' char(file_names(i))]) ;
    copyfile( [current_path '\' dir_name '\include\'  char(file_names(i))], ...
              [current_path '\' dir_name '\src\osqp\' char(file_names(i))]) ;
end