function [mpc_cost, mpc_constraints] = gen_mpc_matrices(problem)

N = problem.system.N;
n = problem.system.n;
m = problem.system.m;
Sx = problem.system.Sx;
Su = problem.system.Su;
%x = Su * u + Sx * x0


if (problem.system.stable)
    C_tilde = [kron(eye(N), problem.constraints.C); 
                 zeros(size(problem.constraints.G,1), n * (N-1)) problem.constraints.G];
    D_tilde = [kron(eye(N), problem.constraints.D); zeros(size(problem.constraints.G,1), m*N)];
    E_tilde = [kron(eye(N), problem.constraints.E); zeros(size(problem.constraints.G,1), N)];
    mpc_constraints.bin = [kron(ones(N,1),problem.constraints.e); problem.constraints.h];
    Q_big = blkdiag(kron(eye(N-1), problem.cost.Q), problem.cost.P);
  
else
    C_tilde = kron(eye(N), problem.constraints.C);
    D_tilde = kron(eye(N), problem.constraints.D);
    E_tilde = kron(eye(N), problem.constraints.E);
    mpc_constraints.bin = kron(ones(N,1),problem.constraints.e);
    Q_big = blkdiag(kron(eye(N), problem.cost.Q));
end


mpc_constraints.Ain = [C_tilde * Su + D_tilde E_tilde];
mpc_constraints.cin = -C_tilde * Sx; 
    
% Cost

if(problem.system.stable)
    Q_big = blkdiag(kron(eye(N-1), problem.cost.Q), problem.cost.P);
else
    Q_big = blkdiag(kron(eye(N), problem.cost.Q));
end    

R_big = kron(eye(problem.system.N), problem.cost.R);
Q_R_tilde = Su' * Q_big * Su + R_big;
%slack 
S_bar = kron(eye(N), problem.cost.S);
H = [Q_R_tilde zeros(size(Q_R_tilde,1), size(S_bar,2)) ;
     zeros(size(S_bar,1), size(Q_R_tilde,1)) S_bar ];
f = [2 * Sx' * Q_big * Su zeros(n, N)];

mpc_cost.H = 2 * H;
mpc_cost.f = f;


