function optimal = online_calc(problem,x)

% get optimal decision variable and optimal value
% OSQP: min   0.5 x' * H * x + (x_0' * f)' * x
%      s.t   -Inf \leq Ain \leq bin + cin * x0


N   = problem.system.N;
n   = problem.system.n;
m   = problem.system.m;

H   = problem.mpc_cost.H;
f   = problem.mpc_cost.f;
q   = (x' * f)';
A   = problem.mpc_constraints.Ain;
bin = problem.mpc_constraints.bin;
cin = problem.mpc_constraints.cin;

ub = bin + cin * x;
lb = -Inf * ones(size(ub,1),1); 

%% osqp
qp_problem         = osqp;                            % create osqp object
settings           = qp_problem.default_settings();   % obtain defualt settings
settings.eps_abs   = 1e-04;
settings.eps_rel   = 1e-04;
settings.verbose   = 0;                               % disable printing output              
qp_problem.setup(H, ...
                 q, ...             
                 A, lb, ub, ...
                 settings);             
output = qp_problem.solve();                          % solve
%% extract x, u
% x = Su u * Sx * x0

u = output.x(1:N*m);
z = zeros(n,N);
v = zeros(m,N);

z_tmp = problem.system.Su * u + problem.system.Sx * x;
v_tmp = u;
for i = 1:problem.system.N
    z(:,i) = z_tmp((i-1)*n + 1:i*n);
    v(:,i) = v_tmp((i-1)*m + 1:i*m);
end    

%% return 
optimal.x        = z;                          % optimal state trajectory
optimal.u        = v;                          % optimal control trajectory
optimal.e        = output.info.status_val;     % status value
optimal.run_time = output.info.run_time;       % run time
optimal.eps      = output.x(N*m+1:N*(m+1));    % slack variables
